import datetime as dt

from emitime import Moment
from pytest import mark


def test_moment_by_pydate() -> None:
    d = dt.date(year=2014, month=3, day=16)
    m = Moment(d)
    assert m.datetime.year == d.year
    assert m.datetime.month == d.month
    assert m.datetime.day == d.day
    assert m == d


@mark.parametrize(
    "value",
    [
        "2014-03-16",
        "2014/03/16",
        "16.03.2014",
    ],
)
def test_moment_by_string(value: str) -> None:
    m = Moment(value)
    assert m.datetime.year == 2014
    assert m.datetime.month == 3
    assert m.datetime.day == 16


@mark.parametrize(
    "date,target",
    [
        (dt.datetime(1992, 1, 2), "1992-01-02"),
    ],
)
def test_moment_to_string(date: dt.datetime, target: str) -> None:
    assert Moment(date) == target


def test_moment_comparison():
    m1 = Moment(dt.date(year=1992, month=1, day=2))
    m2 = Moment(dt.date(year=1993, month=12, day=12))
    m3 = Moment(dt.date(year=1993, month=12, day=12))
    assert m1 < m2 <= m3
    assert m3 >= m2 > m1
    assert m3 >= m1
    assert m1 != m2
    assert m2 == m3
    assert m1 < m2.datetime
    assert m2.datetime > m1


def test_moment_arithmetic():
    d1 = dt.date(year=1992, month=1, day=2)
    d2 = dt.date(year=1993, month=12, day=12)
    assert Moment(d2) - Moment(d1) == d2 - d1
    assert Moment(d2) - d1 == d2 - d1
    assert d2 - Moment(d1) == d2 - d1



__all__ = ("momentrange",)

from .base import Interval, Moment
import typing as tp


def momentrange(start, stop, step) -> tp.Iterable[Moment]:
    start = Moment(start).datetime
    stop = Moment(stop).datetime
    step = Interval(step).timedelta
    assert stop != start
    side = True
    if step.total_seconds() > 0.001:
        assert start < stop
    elif step.total_seconds() < -0.001:
        side = False
        assert stop < start
    else:
        raise ValueError(step)
    while start < stop if side else start > stop:
        yield Moment(start)
        start = start + step



from emitime.base import Interval, Moment
from typing import Any, Callable

class ParserRAM:
    def __init__(self, factory: Callable) -> None: ...
    def __call__(self, *args, **kwargs) -> Any: ...

class MomentParserRAM(ParserRAM):
    def __init__(self) -> None: ...
    def __call__(self, *args, **kwargs) -> Moment: ...

class IntervalParserRAM(ParserRAM):
    def __init__(self) -> None: ...
    def __call__(self, *args, **kwargs) -> Interval: ...



__all__ = (
    "MomentParserRAM",
    "IntervalParserRAM"
)

import pickle
from typing import Any, Callable, Dict

import xxhash
from emitime.base import Interval, Moment


class ParserRAM:
    def __init__(self, factory: Callable):
        self._mem: Dict[bytes, Any] = dict()
        self._hash = xxhash.xxh3_128_digest
        self._make = factory

    def __call__(self, *args, **kwargs) -> Any:
        dump = pickle.dumps(args) + pickle.dumps(kwargs)
        hash = self._hash(dump)
        if hash in self._mem:
            return self._mem[hash]
        obj = Moment(*args, **kwargs)
        self._mem[hash] = obj
        return obj


class MomentParserRAM(ParserRAM):
    def __init__(self):
        super().__init__(Moment)

    def __call__(self, *args, **kwargs) -> Moment:
        return super().__call__(*args, **kwargs)


class IntervalParserRAM(ParserRAM):
    def __init__(self):
        super().__init__(Interval)

    def __call__(self, *args, **kwargs) -> Interval:
        return super().__call__(*args, **kwargs)



import runpy
from pathlib import Path

from setuptools import find_packages, setup

name = "emitime"
descriprion = "Emit time from any time!"
author = "Vladislav A. Proskurov"
author_email = "rilshok@pm.me"
classifiers = [
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.6",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
]

version_path = Path(__file__).resolve().parent / name / "__version__.py"
version = runpy.run_path(str(version_path))["__version__"]

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", encoding="utf-8") as file:
    requirements = file.read().splitlines()

setup(
    name=name,
    descriprion=descriprion,
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    author=author,
    author_email=author_email,
    version=version,
    url="https://github.com/rilshok/emitime",
    packages=find_packages(include=(name,)),
    include_package_data=True,
    zip_safe=False,
    install_requires=requirements,
    classifiers=classifiers,
    python_requires=">=3.6",
)
